@@ -168,12 +168,11 @@ void ColumnCommand::_loadData()
 
 
     _mask = mask;
-// 	primMsg->RidFlags = 0xffff;   // disables selective block loading
-    //cout <<__FILE__ << "::issuePrimitive() o: " << getOID() << " l:" << primMsg->LBID << " ll: " << oidLastLbid << endl;
+ 	//primMsg->RidFlags = 0xffff;   // disables selective block loading
+    //cerr << "::ColumnCommand::_loadData OID " << getOID() << " l:" << primMsg->LBID << " ll: " << oidLastLbid << " primMsg->RidFlags " << primMsg->RidFlags << endl;
 
     for (i = 0; i < W; ++i, _mask <<= shift)
     {
-
         if ((!lastBlockReached && _isScan) || (!_isScan && primMsg->RidFlags & _mask))
         {
             lbids[blocksToLoad] = primMsg->LBID + i;
@@ -397,7 +396,6 @@ void ColumnCommand::_process_OT_BOTH()
 {
     using T = typename datatypes::WidthToSIntegralType<W>::type;
     bpp->ridCount = outMsg->NVALS;
-    bpp->ridCount = outMsg->NVALS;
     bpp->ridMap = outMsg->RidFlags;
     uint8_t* outPtr = reinterpret_cast<uint8_t*>(&outMsg[1]);
     auto* ridPos = primitives::getRIDArrayPosition(outPtr, 0);
